package semfly.tests.agents;

import semfly.agents.ITPQOwner;
import semfly.agents.behaviours.CoordinatorReceiverBL;
import semfly.log.LogWriter;
import semfly.onto.jade.RequestAction;
import semfly.onto.jade.RequestToken;
import semfly.onto.jade.SaveAction;
import semfly.onto.jade.SaveInDB;
import semfly.onto.jade.SaveInTPQ;
import semfly.onto.jade.TokensOntology;
import jade.content.lang.sl.SLCodec;
import jade.core.behaviours.OneShotBehaviour;
import jade.core.Agent;
import jade.lang.acl.ACLMessage;
import semfly.onto.jade.IndexToken;
import jade.content.AgentAction;

/**
 * @author Szymon Pisarek (szymekpi@o2.pl)
 *
 * This class is responsible for testing CoordinatorReceiverBL class.
 */
public class TestCoordinatorReceiverBL extends OneShotBehaviour
{
    /**
     * This class is tested by this behaviour.
     */
    private CoordinatorReceiverBL receiverBL;
    
    private boolean testSucceeded = true;
    
    public TestCoordinatorReceiverBL(Agent agent)
    {
        receiverBL = new CoordinatorReceiverBL(agent);
    }
    
    /**
     * Runs tests.
     */
    public void action()
    {
       testNullAction();
       
       testSaveAction();
       testSaveTPQ_NullToken();
       testSaveTPQ_RightToken();
       
       testRequestAction();
       testRequestToken();
       
       testIsActionForMe();
       
       System.out.print("TestCoordinatorReceiverBL - TESTS ");
       if (testSucceeded) System.out.println("SUCCEEDED!!!");
       else System.out.println("FAILED!!!");
    }

    /**
     * Tests performing "Action" by using null referenc.
     */
    private void testNullAction()
    {
        try
        {
            receiverBL.performAction(null, null);
            testSucceeded = false;
            LogWriter.writeWarn2Log("INFO", 
                    "testNullAction() - IllegalArgumentException expected");
        }
        catch (Exception ex) {}
    }
    
    /**
     * Tests performing "Save Action" by using SaveAction class.
     */
    private void testSaveAction()
    {
        try
        {
            receiverBL.performAction(new SaveAction(), null);
            testSucceeded = false;
            LogWriter.writeWarn2Log("INFO", 
                    "testSaveAction() - AgentException expected");
        }
        catch (Exception ex) {}
    }
    
    /**
     * Test performing "Save in TPQ" action by using null token.
     */
    private void testSaveTPQ_NullToken()
    {
        try
        {
            SaveInTPQ saveAction = new SaveInTPQ();
            saveAction.setIndexToken(null);
            receiverBL.performAction(saveAction, null);
            testSucceeded = false;
            LogWriter.writeWarn2Log("INFO", 
                    "testSaveTPQ_NullToken() - IllegalArgumentException expected");
        }
        catch (Exception ex) { }
    }
    
    /**
     * Test performing "Save in TPQ" action by using right token.
     */
    private void testSaveTPQ_RightToken()
    {
        try
        {
            ((Tester)myAgent).clearTPQ();
            
            SaveInTPQ saveAction = new SaveInTPQ();
            IndexToken token = new IndexToken();
            saveAction.setIndexToken(token);
            receiverBL.performAction(saveAction, 
                    new ACLMessage(ACLMessage.INFORM));
            
            if (((ITPQOwner)myAgent).poolIndexToken() != token)
            {
                testSucceeded = false;
                LogWriter.writeWarn2Log("INFO", 
                    	"testSaveTPQ_RightToken() failed; token != token");
            }
        }
        catch (Exception ex) 
        {
            testSucceeded = false;
            LogWriter.writeWarn2Log("INFO", 
            	"testSaveTPQ_RightToken() failed", ex);
        }
    }
    
    /**
     * Tests performing "Request Action" by using RequestAction class.
     */
    private void testRequestAction()
    {
        try
        {
            receiverBL.performAction(new RequestAction(), null);
            testSucceeded = false;
            LogWriter.writeWarn2Log("INFO", 
                    "testRequestAction() - AgentException expected");
        }
        catch (Exception ex) {}
    }
    
    /**
     * Tests performing "Request Token" action when TPQ holder has no tokens
     * in its TPQ.
     * @author szymek
     */
    private void testRequestToken()
    {
        try
        {
            registerLangAndOnto();
            
            ((Tester)myAgent).clearTPQ();
            IndexToken token = new IndexToken();
            token.setJenaModel("JENA_MODEL");
            ((ITPQOwner)myAgent).addIndexToken2TPQ(token);
            
            ACLMessage msg = createACLMessage();
            receiverBL.performAction(new RequestToken(), msg);
            
            SaveInDB saveDB = (SaveInDB)receiverBL.extractAgentAction(msg);
            if (saveDB.getIndexToken().getJenaModel().equals(token))
            {
                testSucceeded = false;
                LogWriter.writeWarn2Log("INFO", 
            		"testRequestToken() failed; token != token");
            }
        }
        catch (Exception ex) 
        {
            testSucceeded = false;
            LogWriter.writeWarn2Log("INFO", 
            	"testRequestToken() failed", ex);
        }
    }
    
    public void testIsActionForMe()
    {
        try
        {
            AgentAction action = new SaveAction();
            if (receiverBL.isActionForMe(action) == true)
            {
                testSucceeded = false;
                LogWriter.writeWarn2Log("INFO", 
                	"testIsActionForMe() - SaveAction is not for me");
            }
            
            action = new RequestAction();
            if (receiverBL.isActionForMe(action) == true)
            {
                testSucceeded = false;
                LogWriter.writeWarn2Log("INFO", 
                	"testIsActionForMe() - RequestAction is not for me");
            }
            
            action = new SaveInTPQ();
            if (receiverBL.isActionForMe(action) == false)
            {
                testSucceeded = false;
                LogWriter.writeWarn2Log("INFO", 
                	"testIsActionForMe() - RequestAction is for me");
            }
            
            action = new RequestToken();
            ((Tester)myAgent).clearTPQ();
            if (receiverBL.isActionForMe(action) == true)
            {
                testSucceeded = false;
                LogWriter.writeWarn2Log("INFO", 
                	"testIsActionForMe() - RequestAction is not for me" +
                	"when I don't have any tokens");
            }
            
            action = new RequestToken();
            ((ITPQOwner)myAgent).addIndexToken2TPQ(new IndexToken());
            if (receiverBL.isActionForMe(action) == false)
            {
                testSucceeded = false;
                LogWriter.writeWarn2Log("INFO", 
                	"testIsActionForMe() - RequestAction is for me" +
                	"when I have some tokens");
            }
        }
        catch (Exception ex) 
        {
            testSucceeded = false;
            LogWriter.writeWarn2Log("INFO", 
                	"testIsActionForMe() failed", ex);
        }
    }
    
    private ACLMessage createACLMessage()
    {
        ACLMessage msg = new ACLMessage(ACLMessage.INFORM);
        msg.setLanguage((new SLCodec()).getName());
        msg.setOntology(TokensOntology.getInstance().getName());
        
        return msg;
    }
    
    private void registerLangAndOnto()
    {
        ((Tester)myAgent).registerLanguage(new SLCodec());
        ((Tester)myAgent).registerOntology(TokensOntology.getInstance());
    }
}
