package semfly.tests.behaviours;

import semfly.agents.CoordinatorAgent;
import semfly.agents.WrapperAgentInfo;
import semfly.agents.behaviours.CoordinatorSchedulerBL;
import semfly.globals.LongWrapper;
import junit.awtui.TestRunner;
import junit.framework.TestCase;

/**
 * @author Szymon Pisarek (szymekpi@o2.pl)
 *
 * This class is responsible for testing CoordinatorScheduler class.
 */
public class CoordinatorSchedulerTest extends TestCase
{
    /**
     * Test data.
     */
    private AgentsInfoTestData agentsInfoTD;
    
    /**
     * Runs tests.
     */
    public static void main(String[] args)
    {
        TestRunner.run(CoordinatorSchedulerTest.class);
    }
    
    /**
     * Creates test data.
     */
    protected void setUp()
    {
        agentsInfoTD = new AgentsInfoTestData();
    }
    
    protected void tearDown()
    {
    }
    
    /**
     * Tests geting the next group of agents to start by using random agents 
     * schedule (test data - group A).
     */
    public void testGetNextAgentsRandomScheduleA()
    {
        CoordinatorSchedulerBL scheduler = 
            new CoordinatorSchedulerBL(new CoordinatorAgent());
        
        createRandomScheduleA();
        scheduler.setAgentsSchedule(agentsInfoTD.getAllAgents());
        scheduler.setCurrentTimestamp(agentsInfoTD.getCurrentTimestamp());
        
        LongWrapper sleepTime = new LongWrapper(0),
        			nextTimestamp = new LongWrapper(0);
        WrapperAgentInfo[] nextAgents = 
            scheduler.getNextScheduledAgents(nextTimestamp, sleepTime);
        
        checkGetNextAgents(nextAgents, nextTimestamp.getValue(),
                sleepTime.getValue());
    }
    
    /**
     * Tests geting the next group of agents to start by using random agents 
     * schedule (test data - group B).
     */
    public void testGetNextAgentsRandomScheduleB()
    {
        CoordinatorSchedulerBL scheduler = 
            new CoordinatorSchedulerBL(new CoordinatorAgent());
        
        createRandomScheduleB();
        scheduler.setAgentsSchedule(agentsInfoTD.getAllAgents());
        scheduler.setCurrentTimestamp(agentsInfoTD.getCurrentTimestamp());
        
        LongWrapper sleepTime = new LongWrapper(0),
        			nextTimestamp = new LongWrapper(0);
        WrapperAgentInfo[] nextAgents = 
            scheduler.getNextScheduledAgents(nextTimestamp, sleepTime);
        
        checkGetNextAgents(nextAgents, nextTimestamp.getValue(),
                sleepTime.getValue());
    }
    
    /**
     * Tests getting the next group of agents to start by using null agents 
     * schedule.
     */
    public void testGetNextAgentsNullSchedule()
    {
        try
        {
	        CoordinatorSchedulerBL scheduler = 
	            new CoordinatorSchedulerBL(new CoordinatorAgent());
	        
	        scheduler.setAgentsSchedule(null);
	        scheduler.setCurrentTimestamp(0);
	        scheduler.getNextScheduledAgents(
	                new LongWrapper(0), new LongWrapper(0));
	        
	        fail("IllegalArgumentException expected");
	    }
        catch (IllegalArgumentException ex) {}
    }
    
    /**
     * Tests getting the next group of agents to start by using empty agents 
     * schedule.
     */
    public void testGetNextAgentsEmptySchedule()
    {
        try
        {
	        CoordinatorSchedulerBL scheduler = 
	            new CoordinatorSchedulerBL(new CoordinatorAgent());
	        
	        scheduler.setAgentsSchedule(new WrapperAgentInfo[0][0]);
	        scheduler.setCurrentTimestamp(0);
	        scheduler.getNextScheduledAgents(
	                new LongWrapper(0), new LongWrapper(0));
	        
	        fail("IllegalArgumentException expected");
	    }
        catch (IllegalArgumentException ex) {}
    }
    
    /**
     * Tests getting the next group of agents to start by using a negative
     * current timestamp.
     */
    public void testGetNextAgentsNegativeTimestamp()
    {
        try
        {
	        CoordinatorSchedulerBL scheduler = 
	            new CoordinatorSchedulerBL(new CoordinatorAgent());
	        
	        scheduler.setAgentsSchedule(new WrapperAgentInfo[1][1]);
	        scheduler.setCurrentTimestamp(-1);
	        scheduler.getNextScheduledAgents(
	                new LongWrapper(0), new LongWrapper(0));
	        
	        fail("IllegalArgumentException expected");
	    }
        catch (IllegalArgumentException ex) {}
    }
    
    private void checkGetNextAgents(WrapperAgentInfo[] nextAgents, 
            long nextTimestamp, long sleepTime)
    {
        assertEquals("nextTimestamp", agentsInfoTD.getNextTimestamp(), 
                nextTimestamp);
        assertEquals("SleepTime", agentsInfoTD.getSleepTime(), sleepTime);
        assertEquals(agentsInfoTD.getNextAgents().length, 
                nextAgents.length);
        
        for (int i = 0; i < nextAgents.length; i++)
            assertEquals(agentsInfoTD.getNextAgents()[i].getTimestamp(),
                    nextAgents[i].getTimestamp());
    }
    
    protected void createRandomScheduleA()
    {
        long currTimestamp = 25;
        long nextTimestamp = 30;
        long sleepTime = 5;
        
        WrapperAgentInfo[][] allAgents = 
            new WrapperAgentInfo[][]{ 
	                { new WrapperAgentInfo("agent_1a", null, null, 10),
	                    new WrapperAgentInfo("agent_1b", null, null, 10) },
	                { new WrapperAgentInfo("agent_2a", null, null, 20),
	                    new WrapperAgentInfo("agent_2b", null, null, 20)},
	                { new WrapperAgentInfo("agent_3a", null, null, 30),
	                    new WrapperAgentInfo("agent_3b", null, null, 30)},
	                { new WrapperAgentInfo("agent_4a", null, null, 40),
	                    new WrapperAgentInfo("agent_4b", null, null, 40)},
	                { new WrapperAgentInfo("agent_5a", null, null, 50),
	                    new WrapperAgentInfo("agent_5b", null, null, 50) }
                };
        
        WrapperAgentInfo[] nextAgents = 
            new WrapperAgentInfo[]{
                new WrapperAgentInfo("agent_1a", null, null, 10),
                new WrapperAgentInfo("agent_2a", null, null, 10),
                new WrapperAgentInfo("agent_3a", null, null, 30),
                new WrapperAgentInfo("agent_3b", null, null, 30)
                };
        
        agentsInfoTD.setAllAgents(allAgents);
        agentsInfoTD.setNextAgents(nextAgents);
        agentsInfoTD.setCurrentTimestamp(currTimestamp);
        agentsInfoTD.setNextTimestamp(nextTimestamp);
        agentsInfoTD.setSleepTime(sleepTime);
    }
    
    protected void createRandomScheduleB()
    {
        long currTimestamp = 4;
        long nextTimestamp = 5;
        long sleepTime = 1;
        
        WrapperAgentInfo[][] allAgents = 
            new WrapperAgentInfo[][]{ 
	                { new WrapperAgentInfo("agent_2a", null, null, 2)},
	                { new WrapperAgentInfo("agent_3a", null, null, 3)},
	                { new WrapperAgentInfo("agent_4a", null, null, 4)},
	                { new WrapperAgentInfo("agent_5a", null, null, 5)},
	                { new WrapperAgentInfo("agent_6a", null, null, 6)}
                };
        
        WrapperAgentInfo[] nextAgents = 
            new WrapperAgentInfo[]{
                new WrapperAgentInfo("agent_5a", null, null, 5)
                };
        
        agentsInfoTD.setAllAgents(allAgents);
        agentsInfoTD.setNextAgents(nextAgents);
        agentsInfoTD.setCurrentTimestamp(currTimestamp);
        agentsInfoTD.setNextTimestamp(nextTimestamp);
        agentsInfoTD.setSleepTime(sleepTime);
    }
    
    /**
     * This class represents test data.
     */
    private class AgentsInfoTestData
    {
        /**
         * Agents that should be started next time.
         */
        private WrapperAgentInfo[] nextAgents;
        
        /**
         * All agents.
         */
        private WrapperAgentInfo[][] allAgents;
        
        private long currentTimestamp;
        private long nextTimestamp;
        private long sleepTime;
        
        
        public WrapperAgentInfo[] getNextAgents()
        {
            return nextAgents;
        }
        
        public void setNextAgents(WrapperAgentInfo[] allAgents)
        {
            this.nextAgents = allAgents;
        }
        
        public WrapperAgentInfo[][] getAllAgents()
        {
            return allAgents;
        }
        
        public void setAllAgents(WrapperAgentInfo[][] sortedAgents)
        {
            this.allAgents = sortedAgents;
        }
        
        public long getCurrentTimestamp()
        {
            return currentTimestamp;
        }
        
        public void setCurrentTimestamp(long currentTimestamp)
        {
            this.currentTimestamp = currentTimestamp;
        }
        
        public long getSleepTime()
        {
            return sleepTime;
        }
        
        public void setSleepTime(long sleepTime)
        {
            this.sleepTime = sleepTime;
        }
        
        public long getNextTimestamp()
        {
            return nextTimestamp;
        }
        
        public void setNextTimestamp(long nextTimestamp)
        {
            this.nextTimestamp = nextTimestamp;
        }
    }
}
