package semfly.tests.behaviours;

import jade.core.Agent;
import semfly.agents.behaviours.MarriottHotelCodesLoaderBL;
import semfly.agents.behaviours.MarriottInfoLoaderBL;
import semfly.agents.behaviours.MarriottLocationsLoaderBL;
import semfly.dto.HotelLocation;
import semfly.dto.WebPageDocument;
import semfly.exceptions.WebSourceLoadException;
import semfly.exceptions.WebSourceParseException;
import semfly.net.WebSiteLoader;
import junit.awtui.TestRunner;
import junit.framework.TestCase;

import org.apache.commons.httpclient.methods.GetMethod;
import org.w3c.dom.Document;

/**
 * @author Szymon Pisarek (szymekpi@o2.pl)
 *
 * This class is responsible for testing Marriott Wrapper behaviours.
 */
public class MarriottBehavioursTest extends TestCase
{

    public static void main(String[] args)
    {
        TestRunner.run(MarriottBehavioursTest.class);
    }

   /**
    * Tests for MarriottLocationsLoaderBL class.
    */
    
    public void testLoadLocation_ValidPageURI()
    {
        MarriottLocationsLoaderBL marriottLoader = 
            new MarriottLocationsLoaderBL(new Agent());
        
        try
        {
            marriottLoader.loadLocations();
        }
        catch (Exception ex)
        {
            assertTrue("Caught exception: " + ex.toString(), false);
        }
    }
    
    public void testLoadLocation_InvalidPageURI()
    {
        MarriottLocationsLoaderBL marriottLoader = 
            new MarriottLocationsLoaderBL(new Agent());
        
        marriottLoader.setPageURI("URI");
        
        try
        {
            marriottLoader.loadLocations();
            fail("WebSourceLoadException expected");
        }
        catch (WebSourceLoadException ex) {}
    }
    
    /**
     * Tries to extract hotels location codes from page that is not related 
     * with any hotel, like www.onet.pl.
     */
    public void testExtractLocationsCodes_InvalidPageURI()
    {
        MarriottLocationsLoaderBL marriottLoader = 
            new MarriottLocationsLoaderBL(new Agent());
        
        marriottLoader.setPageURI("http://www.onet.pl");
        
        try
        {
            WebPageDocument doc = marriottLoader.loadLocations();
            marriottLoader.extractLocationCodes(doc.getWebDocument());
            fail("WebSourceParseException expected");
        }
        catch (WebSourceLoadException ex)
        {
            assertTrue("Caught exception: " + ex.toString(), false);
        }
        catch (WebSourceParseException ex) {}
    }
    
    /**
     * Tries to extract hotels location codes from www.marriott.com.
     */
    public void testExtractLocationsCodes_ValidPageURI()
    {
        MarriottLocationsLoaderBL marriottLoader = 
            new MarriottLocationsLoaderBL(new Agent());
        
        try
        {
            WebPageDocument doc = marriottLoader.loadLocations();
            marriottLoader.extractLocationCodes(doc.getWebDocument());
        }
        catch (Exception ex)
        {
            assertTrue("Caught exception: " + ex.toString(), false);
        }
    }
    
    /**
     * Tests for MarriottHotelCodesLoaderBL class.
     */
    public void testLoadHotelsCodes_ValidPageURI()
    {
        HotelLocation[] hotelsLocations = new HotelLocation[]{
            new HotelLocation("BR", HotelLocation.Country),
            new HotelLocation("EG", HotelLocation.Country),
            new HotelLocation("DE", HotelLocation.USState),
            new HotelLocation("OH", HotelLocation.USState)
        };
        
        MarriottHotelCodesLoaderBL marriottLoader = 
            new MarriottHotelCodesLoaderBL(new Agent(), hotelsLocations);
        
        try
        {
            marriottLoader.loadResources();
        }
        catch (Exception ex)
        {
            assertTrue("Caught exception: " + ex.toString(), false);
        }
    }
    
    public void testLoadHotelsCodes_InvalidPageURI()
    {
        HotelLocation[] hotelsLocations = new HotelLocation[]{
            new HotelLocation("BUEAR", HotelLocation.Country),
            new HotelLocation("VIESE", HotelLocation.Country),
            new HotelLocation("BHMSC", HotelLocation.USState),
            new HotelLocation("PVDFI", HotelLocation.USState)
        };
        
        MarriottHotelCodesLoaderBL marriottLoader = 
            new MarriottHotelCodesLoaderBL(new Agent(), hotelsLocations);
        
        marriottLoader.setBaseWebSiteURI("URI");
        try
        {
            marriottLoader.loadResources();
            fail("WebSourceLoadException expected");
        }
        catch (WebSourceLoadException ex) {}
    }
    
    public void testExtractHotelsCodes_ValidPages()
    {
        HotelLocation[] hotelsLocations = new HotelLocation[]{
                new HotelLocation("BR", HotelLocation.Country),
                new HotelLocation("EG", HotelLocation.Country),
                new HotelLocation("DE", HotelLocation.USState),
                new HotelLocation("OH", HotelLocation.USState)
            };
            
        MarriottHotelCodesLoaderBL marriottLoader = 
            new MarriottHotelCodesLoaderBL(new Agent(), hotelsLocations);
        
        try
        {
            Document[] docs = marriottLoader.loadResources();
            marriottLoader.extractHotelsCodes(docs);
        }
        catch (Exception ex)
        {
            assertTrue("Caught exception: " + ex.toString(), false);
        }
    }
    
    public void testExtractHotelsCodes_InvalidPages()
    {
        MarriottHotelCodesLoaderBL marriottLoader = 
            new MarriottHotelCodesLoaderBL(new Agent(), new HotelLocation[]{});
        
        try
        {
            marriottLoader.loadResources();
            GetMethod httpMethod = new GetMethod("http://www.onet.pl");
            WebSiteLoader webLoader = new WebSiteLoader();
    		Document[] docs = 
    		    new Document[]{webLoader.loadWebSiteHTML(httpMethod)};
    		
            marriottLoader.extractHotelsCodes(docs);
            fail("WebSourceParseException expected");
        }
        catch (WebSourceParseException ex) {}
        catch (Exception ex)
        {
            assertTrue("Caught exception: " + ex.toString(), false);
        }
    }
    
    /**
     * Tests for MarriottInfoLoaderBL class.
     */
    public void testLoadHotelsInfo_ValidPageURI()
    {
        MarriottInfoLoaderBL marriottLoader = 
            new MarriottInfoLoaderBL(new Agent(), new String[]{});
        
        try
        {
            marriottLoader.loadResources();
        }
        catch (Exception ex)
        {
            assertTrue("Caught exception: " + ex.toString(), false);
        }
    }
    
    public void testLoadHotelsInfo_InvalidPageURI()
    {
        MarriottInfoLoaderBL marriottLoader = 
            new MarriottInfoLoaderBL(new Agent(), new String[]{"AAA"});
        
        marriottLoader.setWebSiteURI("URI");
        try
        {
            marriottLoader.loadResources();
            fail("WebSourceLoadException expected");
        }
        catch (WebSourceLoadException ex) {}
    }
}
