package semfly.tests.behaviours;

import jade.core.Agent;
import org.apache.commons.httpclient.methods.GetMethod;
import semfly.agents.behaviours.SheratonHotelCodesLoaderBL;
import semfly.agents.behaviours.SheratonInfoLoaderBL;
import semfly.agents.behaviours.SheratonLocationsLoaderBL;
import semfly.dto.HotelLocation;
import semfly.dto.SheratonHotelCode;
import semfly.dto.WebPageDocument;
import semfly.exceptions.WebSourceLoadException;
import semfly.exceptions.WebSourceParseException;
import semfly.net.WebSiteLoader;
import junit.awtui.TestRunner;
import junit.framework.TestCase;

/**
 * @author Szymon Pisarek (szymekpi@o2.pl)
 *
 * This class is responsible for testing Marriott Wrapper behaviours.
 */
public class SheratonBehavioursTest extends TestCase
{

    public static void main(String[] args)
    {
        TestRunner.run(SheratonBehavioursTest.class);
    }

    /**
     * Tests for SheratonLocationsLoaderBL class.
     */
     public void testLoadLocation_ValidPageURI()
     {
         SheratonLocationsLoaderBL sheratonLoader = 
             new SheratonLocationsLoaderBL(new Agent());
         
         try
         {
             sheratonLoader.loadLocations();
         }
         catch (Exception ex)
         {
             assertTrue("Caught exception: " + ex.toString(), false);
         }
     }
     
     public void testLoadLocation_InvalidPageURI()
     {
         SheratonLocationsLoaderBL sheratonLoader = 
             new SheratonLocationsLoaderBL(new Agent());
         
         sheratonLoader.setPageURI("URI");
         
         try
         {
             sheratonLoader.loadLocations();
             fail("WebSourceLoadException expected");
         }
         catch (WebSourceLoadException ex) {}
     }
     
     /**
      * Tries to extract hotels location codes from page that is not related 
      * with any hotel, like www.onet.pl.
      */
     public void testExtractLocationsCodes_InvalidPageURI()
     {
         SheratonLocationsLoaderBL sheratonLoader = 
             new SheratonLocationsLoaderBL(new Agent());
         
         sheratonLoader.setPageURI("http://www.onet.pl");
         
         try
         {
             WebPageDocument doc = sheratonLoader.loadLocations();
             sheratonLoader.extractLocationCodes(doc.getWebDocument());
             fail("WebSourceParseException expected");
         }
         catch (WebSourceLoadException ex)
         {
             assertTrue("Caught exception: " + ex.toString(), false);
         }
         catch (WebSourceParseException ex) {}
     }
     
     /**
      * Tries to extract hotels location codes from www.marriott.com.
      */
     public void testExtractLocationsCodes_ValidPageURI()
     {
         SheratonLocationsLoaderBL sheratonLoader = 
             new SheratonLocationsLoaderBL(new Agent());
         
         try
         {
             WebPageDocument doc = sheratonLoader.loadLocations();
             sheratonLoader.extractLocationCodes(doc.getWebDocument());
         }
         catch (Exception ex)
         {
             assertTrue("Caught exception: " + ex.toString(), false);
         }
     }
     
     
     /**
      * Tests for SheratonHotelCodesLoaderBL class.
      */
     public void testLoadHotelsCodes_ValidPageURI()
     {
         HotelLocation[] hotelsLocations = new HotelLocation[]{
             new HotelLocation("BR", HotelLocation.Country),
             new HotelLocation("EG", HotelLocation.Country),
             new HotelLocation("DE", HotelLocation.USState),
             new HotelLocation("OH", HotelLocation.USState)
         };
         
         SheratonHotelCodesLoaderBL sheratonLoader = 
             new SheratonHotelCodesLoaderBL(new Agent(), hotelsLocations);
         
         try
         {
             sheratonLoader.loadResources();
         }
         catch (Exception ex)
         {
             assertTrue("Caught exception: " + ex.toString(), false);
         }
     }
     
     public void testLoadHotelsCodes_InvalidPageURI()
     {
         HotelLocation[] hotelsLocations = new HotelLocation[]{
                 new HotelLocation("BR", HotelLocation.Country),
                 new HotelLocation("EG", HotelLocation.Country),
                 new HotelLocation("DE", HotelLocation.USState),
                 new HotelLocation("OH", HotelLocation.USState)
         };
         
         SheratonHotelCodesLoaderBL sheratonLoader = 
             new SheratonHotelCodesLoaderBL(new Agent(), hotelsLocations);
         
         sheratonLoader.setFirstBaseWebSiteURI("URI");
         sheratonLoader.setSecondBaseWebSiteURI("URI");
         try
         {
             sheratonLoader.loadResources();
             fail("WebSourceLoadException expected");
         }
         catch (WebSourceLoadException ex) {}
     }
     
     public void testExtractHotelsCodes_ValidPages()
     {
         HotelLocation[] hotelsLocations = new HotelLocation[]{
                 new HotelLocation("BR", HotelLocation.Country),
                 new HotelLocation("EG", HotelLocation.Country),
                 new HotelLocation("DE", HotelLocation.USState),
                 new HotelLocation("OH", HotelLocation.USState)
             };
             
         SheratonHotelCodesLoaderBL sheratonLoader = 
             new SheratonHotelCodesLoaderBL(new Agent(), hotelsLocations);
         
         try
         {
             WebPageDocument[] docs = sheratonLoader.loadResources();
             sheratonLoader.extractHotelsCodes(docs);
         }
         catch (Exception ex)
         {
             assertTrue("Caught exception: " + ex.toString(), false);
         }
     }
     
     public void testExtractHotelsCodes_InvalidPages()
     {
         SheratonHotelCodesLoaderBL sheratonLoader = 
             new SheratonHotelCodesLoaderBL(new Agent(), new HotelLocation[]{});
         
         try
         {
             sheratonLoader.loadResources();
             GetMethod httpMethod = new GetMethod("http://www.onet.pl");
             WebSiteLoader webLoader = new WebSiteLoader();
     		 WebPageDocument[] docs = 
     		    new WebPageDocument[]{
     		         new WebPageDocument(webLoader.loadWebSiteHTML(httpMethod))};
     		
     		 sheratonLoader.extractHotelsCodes(docs);
             fail("WebSourceParseException expected");
         }
         catch (WebSourceParseException ex) {}
         catch (Exception ex)
         {
             assertTrue("Caught exception: " + ex.toString(), false);
         }
     }
     
     /**
      * Tests for SheratonInfoLoaderBL class.
      */
     public void testLoadHotelsInfo_ValidPageURI()
     {
         SheratonHotelCode shCode = new SheratonHotelCode();
         shCode.setAffilationCode("aa");
         shCode.setBrand("bb");
         shCode.setChainCode("cc");
         shCode.setPropertyID("pp");
         
         SheratonInfoLoaderBL sheratonLoader = 
             new SheratonInfoLoaderBL(new Agent(), 
             new SheratonHotelCode[]{shCode});
         
         try
         {
             sheratonLoader.loadResources();
         }
         catch (Exception ex)
         {
             assertTrue("Caught exception: " + ex.toString(), false);
         }
     }
     
     public void testLoadHotelsInfo_InvalidPageURI()
     {
         SheratonInfoLoaderBL sheratonLoader = 
             new SheratonInfoLoaderBL(new Agent(), new SheratonHotelCode[]{});
         
         sheratonLoader.setWebSiteURIBegining("URI");
         try
         {
             System.out.println("test");
             sheratonLoader.loadResources();
             fail("WebSourceLoadException expected");
         }
         catch (WebSourceLoadException ex) {}
     }
}
