package semfly.tests.coordination;

import semfly.agents.WrapperAgentInfo;
import semfly.coordination.ScheduleReader;
import junit.framework.TestCase;

/**
 * @author Szymon Pisarek (szymekpi@o2.pl)
 *
 * This class is responsible for testing ScheduleReader class.
 */
public class ScheduleReaderTest extends TestCase
{
    /**
     * Test data.
     */
    private AgentsInfoTestData agentsInfoTD;
    
    /**
     * Creates test data.
     */
    protected void setUp()
    {
        agentsInfoTD = new AgentsInfoTestData();
    }
    
    protected void tearDown()
    {
    }
    
    /**
     * Test agents sorting when agents are in a random order as an input.
     */
    public void testSortByRandomSchedule()
    {
        createRandomSchedule(); 
        WrapperAgentInfo[][] sortedAgents = 
            sortWrappersBySchedule(agentsInfoTD.getAllAgents());
        
        checkSortByScheduleResults(agentsInfoTD.getSortedAgents(), sortedAgents);
    }
    
    /**
     * Test agents sorting when there are no agents as an input.
     */
    public void testSortByEmptySchedule()
    {
        createEmptySchedule();
        WrapperAgentInfo[][] sortedAgents = 
            sortWrappersBySchedule(agentsInfoTD.getAllAgents());
        
        checkSortByScheduleResults(agentsInfoTD.getSortedAgents(), sortedAgents);
    }
    
    /**
     * Test agents sorting when there is a null reference as an input.
     */
    public void testSortByNullSchedule()
    {
        try
        {
            sortWrappersBySchedule(null);
            fail("IllegalArgumentException expected");
        }
        catch (IllegalArgumentException expected) {}
    }
    
    /**
     * Sorts the agents given by schedule.
     */
    private WrapperAgentInfo[][] sortWrappersBySchedule(
            					WrapperAgentInfo[] agentsInfo)
    {
        ScheduleReader scheduleReader = new ScheduleReader();
        return scheduleReader.sortWrappersBySchedule(agentsInfo);
    }
    
    /**
     * Checks agents sorting by comparing agents correctly sorted 
     * with just sorted ones.
     */
    private void checkSortByScheduleResults(
            		WrapperAgentInfo[][] correctlySortedAgents,
            		WrapperAgentInfo[][] sortedAgents)
    {
        checkQuantity(correctlySortedAgents, sortedAgents);
        checkSortOrder(correctlySortedAgents, sortedAgents);
    }
   
    private void checkQuantity(WrapperAgentInfo[][] corrSortedAgents,
            		WrapperAgentInfo[][] sortedAgents)
    {
        assertEquals(corrSortedAgents.length, sortedAgents.length);
        
        int len = corrSortedAgents.length;
        for (int i = 0; i < len; i++)
        {
            assertEquals(corrSortedAgents[i].length, sortedAgents[i].length);
        }
    }
    
    private void checkSortOrder(WrapperAgentInfo[][] corrSortedAgents,
            		WrapperAgentInfo[][] sortedAgents)
    {
        int len = corrSortedAgents.length;
        for (int i = 0; i < len; i++)
        {
            for (int j = 0; j < corrSortedAgents[i].length; j++)
            {
                assertEquals(corrSortedAgents[i][j].getTimestamp(),
                        sortedAgents[i][j].getTimestamp());
            }
        }
    }
    
    private AgentsInfoTestData createRandomSchedule()
    {
        WrapperAgentInfo[] allAgents = 
            new WrapperAgentInfo[]{
                new WrapperAgentInfo("agent_1a", null, null, 10000),
                new WrapperAgentInfo("agent_2a", null, null, 20000),
                new WrapperAgentInfo("agent_3a", null, null, 30000),
                new WrapperAgentInfo("agent_4a", null, null, 40000),
                new WrapperAgentInfo("agent_2b", null, null, 20000),
                new WrapperAgentInfo("agent_6a", null, null, 60000),
                new WrapperAgentInfo("agent_5a", null, null, 50000),
                new WrapperAgentInfo("agent_5b", null, null, 50000),
                new WrapperAgentInfo("agent_3b", null, null, 30000),
                new WrapperAgentInfo("agent_2c", null, null, 20000),
                };
        
        WrapperAgentInfo[][] sortedAgents = 
            new WrapperAgentInfo[][]{ 
	                { new WrapperAgentInfo("agent_1a", null, null, 10000)},
	                { new WrapperAgentInfo("agent_2a", null, null, 20000), 
	                    new WrapperAgentInfo("agent_2b", null, null, 20000),
	                    new WrapperAgentInfo("agent_2c", null, null, 20000)},
	                { new WrapperAgentInfo("agent_3a", null, null, 30000),
	                    new WrapperAgentInfo("agent_3b", null, null, 30000)},
	                { new WrapperAgentInfo("agent_4a", null, null, 40000) },
	                { new WrapperAgentInfo("agent_5a", null, null, 50000),
	                    new WrapperAgentInfo("agent_5b", null, null, 50000)},
	                {  new WrapperAgentInfo("agent_6a", null, null, 60000) }
                };
        
        agentsInfoTD.setAllAgents(allAgents);
        agentsInfoTD.setSortedAgents(sortedAgents);
        return agentsInfoTD;
    }
    
    public void createEmptySchedule()
    {  
        agentsInfoTD.setAllAgents(new WrapperAgentInfo[]{});
        agentsInfoTD.setSortedAgents(new WrapperAgentInfo[][]{});
    }
    
    /**
     * This class represents test data.
     */
    private class AgentsInfoTestData
    {
        private WrapperAgentInfo[] allAgents;
        private WrapperAgentInfo[][] sortedAgents;
        
        
        public WrapperAgentInfo[] getAllAgents()
        {
            return allAgents;
        }
        
        public void setAllAgents(WrapperAgentInfo[] allAgents)
        {
            this.allAgents = allAgents;
        }
        public WrapperAgentInfo[][] getSortedAgents()
        {
            return sortedAgents;
        }
        public void setSortedAgents(WrapperAgentInfo[][] sortedAgents)
        {
            this.sortedAgents = sortedAgents;
        }
    }
}
