package semfly.tests.db;

import jade.core.AID;

import java.io.StringWriter;
import java.sql.SQLException;
import java.util.Date;
import semfly.db.DBWriter;
import semfly.onto.jade.IndexToken;
import semfly.onto.jena.AdministrativeInfo;
import junit.awtui.TestRunner;
import junit.framework.TestCase;
import com.hp.hpl.jena.db.IDBConnection;
import com.hp.hpl.jena.db.ModelRDB;
import com.hp.hpl.jena.db.DBConnection;
import com.hp.hpl.jena.rdf.model.Model;
import com.hp.hpl.jena.rdf.model.ModelFactory;
import com.hp.hpl.jena.rdf.model.ResIterator;
import com.hp.hpl.jena.rdf.model.Resource;
import com.hp.hpl.jena.shared.JenaException;
import com.hp.hpl.jena.vocabulary.VCARD;

/**
 * @author Szymon Pisarek (szymekpi@o2.pl)
 *
 * This class is responsible for testing DBWriter class.
 */
public class DBWriterTest extends TestCase
{
    protected final String DBModelName = "http://tests";
    protected final String DBDriverClassName = "com.mysql.jdbc.Driver";
    protected final String DB_URL = "jdbc:mysql://localhost/dbtests",  
    				DB_USER = "tester", DB_PASSWD = "test", DB = "MySQL"; 
    
    ModelRDB dbModel;
    DBWriter dbWriter;
   
    /**
     * Runs tests.
     */
    public static void main(String[] args)
    {
        TestRunner.run(DBWriterTest.class);
    }
    
    protected void setUp() throws Exception
    {
        dbModel = getDBModel(createDBConnection());
        dbWriter = new DBWriter(createDBConnection(), DBModelName);
    }
    
    protected void tearDown() throws Exception
    {
        dbModel.remove();
        closeDBConnection();
    }
    
    protected ModelRDB getDBModel(IDBConnection dbConn)
	{
	    ModelRDB model;
        
        if( !dbConn.containsModel(DBModelName) )
        	model = ModelRDB.createModel(dbConn, DBModelName);
        else
        	model = ModelRDB.open(dbConn, DBModelName);
        
        return model;
	}
    
    protected IDBConnection createDBConnection() throws ClassNotFoundException
    {    
        Class.forName(DBDriverClassName);                      
        return new DBConnection(DB_URL, DB_USER, DB_PASSWD, DB);
    }
    
    protected void closeDBConnection() throws SQLException
	{
        IDBConnection dbConn = dbModel.getConnection();
	    if (dbConn != null)
	    {
	       dbConn.close();
	    }
	}
    
    protected Model createEmptyModel()
    {
        return ModelFactory.createDefaultModel();
    }
    
    protected Model createModel(String givenName, String familyName)
    {
        Model model = createEmptyModel();
        
        String fullName = givenName + " " + familyName;
        String personURI    = "http://somewhere/" + givenName + "_" + familyName;
        
        model.createResource(personURI)
	        .addProperty(VCARD.FN, fullName)
	        .addProperty(VCARD.N, 
                 model.createResource("http://test/v_card")
                      .addProperty(VCARD.Given, givenName)
                      .addProperty(VCARD.Family, familyName));
        
        return model;
    }
    
    protected Model createModel(String givenName, String familyName, String email)
    {
        Model model = createEmptyModel();
        
        String fullName = givenName + " " + familyName;
        String personURI    = "http://somewhere/" + givenName + "_" + familyName;
        
        model.createResource(personURI)
	        .addProperty(VCARD.FN, fullName)
	        .addProperty(VCARD.EMAIL, email)
	        .addProperty(VCARD.N, 
                 model.createResource("http://test/v_card")
                      .addProperty(VCARD.Given, givenName)
                      .addProperty(VCARD.Family, familyName));
        
        return model;
    }
    
    protected String getModelAsString(Model m)
	{
		StringWriter writer = new StringWriter();
		m.write(writer);
		
		return writer.toString();
	}
    
    /**
     * Adds admin properties from the token given to the model given.
     */
    protected void addAdminProperties(Model model, IndexToken token)
    {
        Resource subject;
		ResIterator subjectsIter = model.listSubjects();
		boolean justCreated = false, justUpdated = true;
		while (subjectsIter.hasNext())
		{
		    subject = subjectsIter.nextResource();
		    if (subject.isAnon()) continue;
		    
		    subject.addProperty(AdministrativeInfo.createdBy, 
		            token.getCreatedBy().getName());
			subject.addProperty(AdministrativeInfo.createdDate, 
			        token.getCreatedDate());
			
			subject.addProperty(AdministrativeInfo.modifiedBy, 
		            token.getCreatedBy().getName());
			subject.addProperty(AdministrativeInfo.modifiedDate, 
			        token.getCreatedDate());
			
			subject.addProperty(AdministrativeInfo.modificationTryBy, 
		            token.getCreatedBy().getName());
			subject.addProperty(AdministrativeInfo.modificationTryDate, 
			        token.getCreatedDate());
		}
    }
    
    /**
     * Tries to save an invalidate token (nothing is set for the token) in the DB. 
     */
    public void testWriteInvalidateToken()
    {
        IndexToken token = new IndexToken();
        
        try
        {
            dbWriter.write2DB(token);
            fail("IllegalArgumentException expected");
        }
        catch (IllegalArgumentException ex){}
    } 
    
    /**
     * Tries to save correct model in the DB.
     */
    public void testWriteCorrectModel()
    {
        IndexToken token = new IndexToken();
        Model model = createModel("givenName", "familyName");
        
        token.setJenaModel(getModelAsString(model));
        token.setCreatedBy(new AID());
        token.setCreatedDate(new Date());
        
        try
        {
            dbWriter.write2DB(token);
        }
        catch (Exception ex)
        {
            assertTrue("Caught exception: " + ex.toString(), false);
        }
        finally
        {
            dbWriter.dispose();
        }
    }
    
    /**
     * Tries to save an invalidate token - with invalidate Jena Model.
     */
    public void testWriteInvalidateModel()
    {
        IndexToken token = new IndexToken();
        token.setJenaModel("invalidate model string");
        token.setCreatedBy(new AID());
        token.setCreatedDate(new Date());
        
        try
        {
            dbWriter.write2DB(token);
            fail("ParseException expected");
        }
        catch (JenaException expected) {}
    }
    
    /**
     * Tries to save an empty token in the DB.
     */
    public void testWriteEmptyModel()
    {
        IndexToken token = new IndexToken();
        Model model = createEmptyModel();
        token.setJenaModel(getModelAsString(model));
        token.setCreatedBy(new AID());
        token.setCreatedDate(new Date());
        
        try
        {
            dbWriter.write2DB(token);
        }
        catch (Exception ex)
        {
            assertTrue("Caught exception: " + ex.toString(), false);
        }
        finally
        {
            dbWriter.dispose();
        }
        
        assertTrue("Models aren't isomorphic", model.isIsomorphicWith(dbModel));
    }
    
    /**
     * Tries to save a token with administrative properties in the DB.
     */
    public void testWriteCorrectModelAdmin()
    {
        IndexToken token = new IndexToken();
        Model model = createModel("givenName", "familyName");
        token.setCreatedBy(new AID());
        token.setCreatedDate(new Date());
        token.setJenaModel(getModelAsString(model));
        
        try
        {
            dbWriter.write2DB(token);
        }
        catch (Exception ex)
        {
            assertTrue("Caught exception: " + ex.toString(), false);
        }
        finally
        {
            dbWriter.dispose();
        }
        
        addAdminProperties(model, token);
        assertTrue("Models aren't isomorphic", model.isIsomorphicWith(dbModel));
    }
}
